@@ -8,7 +8,6 @@
 #' @docType package
 NULL
 
-
 #' Vectorize a scalar function to work on any R object.
 #' 
 #' Robust alternative to \code{\link{Vectorize}} function that accepts any function with two 
@@ -132,7 +131,7 @@ rows <- function(data,rownums)
     result<-data[rownums,]
   }
   #result<-ifelse(is.null(dim(data)),data[c(rownums)],data[c(rownums),])
-  return(result)
+  return((result))
 }
 
 #'Allows finding the 'length' without knowledge of dimensionality.
@@ -151,9 +150,11 @@ len <- function(data)
 }
 
 
-#' A more versatile form of the T-SQL \code{coalesce()} function.  
-#'
-#' Little more than a wrapper for \code{\link{vectorize}}, allows for duplication of SQL coalesce functionality, certain types of if-else statements, and \code{\link{apply}}/\code{\link{Reduce}} combinations.
+#' A more versatile form of the T-SQL \code{coalesce()} function.
+#' 
+#' Little more than a wrapper for \code{\link{vectorize}}, allows for 
+#' duplication of SQL coalesce functionality, certain types of if-else 
+#' statements, and \code{\link{apply}}/\code{\link{Reduce}} combinations.
 #' 
 #' @param ... an arbitrary number of \code{R} objects
 #' @param fun a two argument function that returns an atomic value
@@ -165,6 +166,9 @@ len <- function(data)
 #' coalesce(df$a,df$b)
 #' # Or even just:
 #' coalesce(df)
+#' # Coalesce can actually use any comparison.  For example, instead of non-NA
+#' # values it could find the max in each row:
+#' cbind(EuStockMarkets,Max=coalesce(EuStockMarkets,fun=function (x,y) if (x>y) x else y))
 coalesce<-function(...,fun=(function (x,y) if(!is.na(x)) x else y))
 {
 
@@ -174,7 +178,9 @@ coalesce<-function(...,fun=(function (x,y) if(!is.na(x)) x else y))
 
 #'A more versatile form of the T-SQL \code{count()} function.
 #'
-#'Implementation of T-SQL \code{count} and Excel \code{COUNTIF} functions.  Shows the total number of elements in any number of data objects altogether or that match a condition.
+#'Implementation of T-SQL \code{count} and Excel \code{COUNTIF} functions. 
+#'Shows the total number of elements in any number of data objects altogether or
+#'that match a condition.
 #'
 #'@param ... an arbitrary number of \code{R} objects
 #'@param condition a 1 argument condition
@@ -191,6 +197,91 @@ count<-function(...,condition=(function (x) TRUE))
   return(result)
 }
 
+#'Applies a function over a rolling window on any data object.
+#'
+#'Simple generalized alternative to \code{\link[zoo]{rollapply}} in package
+#'\code{\link[zoo]{zoo}} with the advantage that it works on any type of data
+#'structure (vector, list, matrix, etc) instead of requiring a \code{zoo}
+#'object.
+#'
+#'@param data any \code{R} object
+#'@param fun the function to evaluate
+#'@param window window width defining the size of the subset available to the
+#'  fun at any given point
+#'@param minimum minimum width of the window.  Will not return results if the
+#'  window is truncated below this value at the end of the data set
+#'@param align whether to align the window right or left
+#'@param ... additional arguments to pass to \code{fun}
+#'@export
+#'@examples
+#'rollApply(1:100,sum,minimum=2,window=2)
+#'rollApply(c(1,2,3),sum)
+#'##6 5 3
+#'rollApply(c(1,2,3,4,5,6,7,8,9),sum)
+#'##45 44 42 39 35 30 24 17  9
+#'rollApply(c(1,2,3,4,5,6,7,8,9),sum,window=2)
+#'##3  5  7  9 11 13 15 17  9
+#'rollApply(list(1,2,3,4,5,6,7,8,9),function(x) sum(unlist(x)),window=2,minimum=2)
+#'##3  5  7  9 11 13 15 17
+#'cbind(women,Rolling3=rollApply(women,fun=function(x) mean(x$weight),window=3,align='right'))
+#'
+rollApply <- function(data,fun,window=len(data),minimum=1,align='left',...)
+{
+  if(minimum>len(data))
+    return()
+  FUN=match.fun(fun)
+  if (align=='left')
+    result<-sapply(1:(len(data)-minimum+1),function (x) FUN(rows(data,x:(min(len(data),(x+window-1)))),...))
+  if (align=='right')
+    result<-sapply(minimum:len(data),function (x) FUN(rows(data,max(1,x-window+1):x),...))
+  return(result)
+}
+
+#'Applies a function row-wise on any data object.
+#'
+#'Essentially functions as a \code{MARGIN=1} \code{\link{apply}} apply but also
+#'works on data objects without 2 dimensions such as lists and vectors.
+#'
+#'@param data any \code{R} object
+#'@param fun the function to evaluate
+#'@param ... additional arguments to pass to \code{fun}
+#'@export
+#'@examples
+#'rowApply(list(1,2,3),function (x) sum(unlist(x)))
+#'df<-data.frame(a=c(1,2,3),b=c(1,2,3))
+#'rowApply(df,sum)
+rowApply<-function(data,fun,...)
+{
+  sapply(1:len(data),function (x) fun(rows(data,x),...))
+}
+
+
+#'Inserts a matrix into another matrix.
+#'
+#'Inserts a matrix or data frame into another matrix or data frame.  The new
+#'rows are placed together at the row index specified.
+#'@param existing table to insert into
+#'@param insert rows to insert
+#'@param r index at which to insert
+#'@export
+#'@examples
+#'df1<-data.frame(a=c(1,2,3),b=c(1,2,3),c=c(1,2,3))
+#'insertRows(df1,data.frame(list('a','a','a')),5)
+#'insertRows(df1,data.frame(list('a','a','a')),4)
+#'insertRows(df1,data.frame(list('a','a','a')),3)
+#'insertRows(df1,data.frame(list('a','a','a')),2)
+#'insertRows(df1,data.frame(list('a','a','a')),1)
+#'insertRows(df1,df1,3)
+insertRows <- function(existing, insert, r) {
+  colnames(insert)<-colnames(existing)
+  result<-rbind(existing,insert)
+  sizeA<-len(existing)
+  sizeB<-len(insert)
+  order1<-c(seq_len(sizeA),rep.int(r,sizeB))
+  order2<-c(rep.int(sizeB+1,sizeA),seq_len(sizeB))
+  return(result[order(order1,order2),,drop=FALSE])
+}
+
 #'A more robust form of the R \code{\link{as}} function.
 #'
 #' Alternative to \code{as} that allows any data object to be converted to any other.  
